@@ -8,10 +8,8 @@ from typing import List, Optional, Tuple
 
 import sqlalchemy as sa
 import sqlparse
-from sqlalchemy.dialects import postgresql
-from sqlalchemy.dialects.postgresql import array
+from sqlalchemy.dialects import postgresql  # noqa
 from sqlalchemy.orm import sessionmaker
-from sqlalchemy.sql import Values
 
 from .constants import (
     BUILTIN_SCHEMAS,
@@ -33,8 +31,8 @@ from .exc import (
 from .node import Node
 from .settings import PG_SSLMODE, PG_SSLROOTCERT, QUERY_CHUNK_SIZE
 from .trigger import CREATE_TRIGGER_TEMPLATE
-from .utils import get_postgres_url
-from .view import CreateIndex, CreateView, DropIndex, DropView
+from .urls import get_postgres_url
+from .view import create_view, drop_view
 
 try:
     import citext  # noqa
@@ -50,6 +48,25 @@ except ImportError:
 logger = logging.getLogger(__name__)
 
 
+class TupleIdentifierType(sa.types.UserDefinedType):
+    cache_ok: bool = True
+
+    def get_col_spec(self, **kw) -> str:
+        return "TID"
+
+    def bind_processor(self, dialect):
+        def process(value):
+            return value
+
+        return process
+
+    def result_processor(self, dialect, coltype):
+        def process(value):
+            return value
+
+        return process
+
+
 class Base(object):
     def __init__(self, database: str, verbose: bool = False, *args, **kwargs):
         """Initialize the base class constructor.
@@ -58,10 +75,11 @@ class Base(object):
             database: The database name
         """
         self.__engine = pg_engine(database, **kwargs)
-        self.__schemas = None
+        self.__schemas: Optional[dict] = None
         # models is a dict of f'{schema}.{table}'
         self.models: dict = {}
         self.__metadata: dict = {}
+        self.__indices: dict = {}
         self.verbose: bool = verbose
 
     def connect(self) -> None:
@@ -93,6 +111,13 @@ class Base(object):
                 f"Invalid user permission {permissions}"
             )
 
+        # Azure usernames are of the form username@host on the SQLAlchemy
+        # engine - engine.url.username@host but stored in
+        # pg_user as just user.
+        # we need to extract the real username from: username@host.
+        host_part: str = self.engine.url.host.split(".")[0]
+        username: str = username.split(f"@{host_part}")[0]
+
         with self.__engine.connect() as conn:
             return (
                 conn.execute(
@@ -118,7 +143,7 @@ class Base(object):
             )
 
     # Tables...
-    def model(self, table: str, schema: str):
+    def model(self, table: str, schema: str) -> dict:
         """Get an SQLAlchemy model representation from a table.
 
         Args:
@@ -142,9 +167,10 @@ class Base(object):
                 )
             model = metadata.tables[name]
             model.append_column(sa.Column("xmin", sa.BigInteger))
+            model.append_column(sa.Column("ctid"), TupleIdentifierType)
             # support SQLQlchemy/Postgres 14 which somehow now reflects
             # the oid column
-            if not "oid" in [column.name for column in model.columns]:
+            if "oid" not in [column.name for column in model.columns]:
                 model.append_column(
                     sa.Column("oid", sa.dialects.postgresql.OID)
                 )
@@ -174,7 +200,7 @@ class Base(object):
         return self.__engine
 
     @property
-    def schemas(self):
+    def schemas(self) -> list:
         """Get the database schema names."""
         if self.__schemas is None:
             self.__schemas = sa.inspect(self.engine).get_schema_names()
@@ -183,6 +209,12 @@ class Base(object):
                     self.__schemas.remove(schema)
         return self.__schemas
 
+    def indices(self, table: str) -> list:
+        """Get the database table indexes."""
+        if table not in self.__indices:
+            self.__indices[table] = sa.inspect(self.engine).get_indexes(table)
+        return self.__indices[table]
+
     def tables(self, schema: str) -> List:
         """:obj:`list` of :obj:`str`: Get all tables.
 
@@ -385,7 +417,9 @@ class Base(object):
         )
 
     # Views...
-    def _primary_keys(self, schema: str, tables: List[str]):
+    def _primary_keys(
+        self, schema: str, tables: List[str]
+    ) -> sa.sql.selectable.Select:
         with warnings.catch_warnings():
             warnings.simplefilter("ignore", category=sa.exc.SAWarning)
             pg_class = self.model("pg_class", "pg_catalog")
@@ -394,6 +428,22 @@ class Base(object):
             pg_namespace = self.model("pg_namespace", "pg_catalog")
 
         alias = pg_class.alias("x")
+        inclause = []
+        for table in tables:
+            pairs = table.split(".")
+            if len(pairs) == 1:
+                inclause.append(
+                    self.__engine.dialect.identifier_preparer.quote(pairs[0])
+                )
+            elif len(pairs) == 2:
+                inclause.append(
+                    f"{pairs[0]}.{self.__engine.dialect.identifier_preparer.quote(pairs[-1])}"
+                )
+            else:
+                raise Exception(
+                    f"cannot determine schema and table from {table}"
+                )
+
         return (
             sa.select(
                 [
@@ -447,19 +497,16 @@ class Base(object):
                             sa.dialects.postgresql.REGCLASS,
                         ),
                         sa.Text,
-                    ).in_(
-                        map(
-                            self.__engine.dialect.identifier_preparer.quote,
-                            tables,
-                        )
-                    ),
+                    ).in_(inclause),
                     pg_attribute.c.attnum == sa.any_(pg_index.c.indkey),
                 ]
             )
             .group_by(pg_index.c.indrelid)
         )
 
-    def _foreign_keys(self, schema: str, tables: List[str]):
+    def _foreign_keys(
+        self, schema: str, tables: List[str]
+    ) -> sa.sql.selectable.Select:
         with warnings.catch_warnings():
             warnings.simplefilter("ignore", category=sa.exc.SAWarning)
             table_constraints = self.model(
@@ -515,122 +562,15 @@ class Base(object):
             .group_by(table_constraints.c.table_name)
         )
 
-    def create_view(self, schema, tables, user_defined_fkey_tables):
-        """
-        View describing primary_keys and foreign_keys for each table
-        with an index on table_name
-
-        This is only called once on bootstrap.
-        It is used within the trigger function to determine what payload
-        values to send to pg_notify.
-
-        Since views cannot be modified, we query the existing view for exiting
-        rows and union this to the next query.
-
-        So if 'specie' was the only row before, and the next query returns
-        'unit' and 'structure', we want to end up with the result below.
-
-        table_name | primary_keys | foreign_keys
-        -----------+--------------+--------------
-        specie     | {id}         | {id, user_id}
-        unit       | {id}         | {id, profile_id}
-        structure  | {id}         | {id}
-        """
-
-        views = sa.inspect(self.engine).get_view_names(schema)
-
-        rows = {}
-        if MATERIALIZED_VIEW in views:
-            for table_name, primary_keys, foreign_keys in self.fetchall(
-                sa.select(["*"]).select_from(
-                    sa.text(f"{schema}.{MATERIALIZED_VIEW}")
-                )
-            ):
-                rows.setdefault(
-                    table_name,
-                    {"primary_keys": set([]), "foreign_keys": set([])},
-                )
-                if primary_keys:
-                    rows[table_name]["primary_keys"] = set(primary_keys)
-                if foreign_keys:
-                    rows[table_name]["foreign_keys"] = set(foreign_keys)
-
-            self.__engine.execute(DropView(schema, MATERIALIZED_VIEW))
-
-        if schema != SCHEMA:
-            for table in set(tables):
-                tables.add(f"{schema}.{table}")
-
-        for table_name, columns in self.fetchall(
-            self._primary_keys(schema, tables)
-        ):
-            rows.setdefault(
-                table_name,
-                {"primary_keys": set([]), "foreign_keys": set([])},
-            )
-            if columns:
-                rows[table_name]["primary_keys"] |= set(columns)
-
-        for table_name, columns in self.fetchall(
-            self._foreign_keys(schema, tables)
-        ):
-            rows.setdefault(
-                table_name,
-                {"primary_keys": set([]), "foreign_keys": set([])},
-            )
-            if columns:
-                rows[table_name]["foreign_keys"] |= set(columns)
-
-        if user_defined_fkey_tables:
-            for table_name, columns in user_defined_fkey_tables.items():
-                rows.setdefault(
-                    table_name,
-                    {"primary_keys": set([]), "foreign_keys": set([])},
-                )
-                if columns:
-                    rows[table_name]["foreign_keys"] |= set(columns)
-
-        if not rows:
-            rows.setdefault(
-                None,
-                {"primary_keys": set([]), "foreign_keys": set([])},
-            )
-
-        statement = sa.select(
-            Values(
-                sa.column("table_name"),
-                sa.column("primary_keys"),
-                sa.column("foreign_keys"),
-            )
-            .data(
-                [
-                    (
-                        table_name,
-                        array(fields["primary_keys"])
-                        if fields.get("primary_keys")
-                        else None,
-                        array(fields.get("foreign_keys"))
-                        if fields.get("foreign_keys")
-                        else None,
-                    )
-                    for table_name, fields in rows.items()
-                ]
-            )
-            .alias("t")
+    def create_view(
+        self, schema: str, tables: list, user_defined_fkey_tables: dict
+    ):
+        create_view(
+            self.engine, schema, tables, user_defined_fkey_tables, self
         )
 
-        logger.debug(f"Creating view: {schema}.{MATERIALIZED_VIEW}")
-        self.__engine.execute(CreateView(schema, MATERIALIZED_VIEW, statement))
-        self.__engine.execute(DropIndex("_idx"))
-        self.__engine.execute(
-            CreateIndex("_idx", schema, MATERIALIZED_VIEW, ["table_name"])
-        )
-        logger.debug(f"Created view: {schema}.{MATERIALIZED_VIEW}")
-
     def drop_view(self, schema: str):
-        logger.debug(f"Dropping view: {schema}.{MATERIALIZED_VIEW}")
-        self.__engine.execute(DropView(schema, MATERIALIZED_VIEW))
-        logger.debug(f"Dropped view: {schema}.{MATERIALIZED_VIEW}")
+        drop_view(self.engine, schema)
 
     # Triggers...
     def create_triggers(
@@ -656,7 +596,7 @@ class Base(object):
                 self.drop_triggers(schema, [table])
                 queries.append(
                     sa.DDL(
-                        f"CREATE TRIGGER {table}_{name} "
+                        f'CREATE TRIGGER "{table}_{name}" '
                         f'AFTER {" OR ".join(tg_op)} ON "{schema}"."{table}" '
                         f"FOR EACH {for_each} EXECUTE PROCEDURE "
                         f"{TRIGGER_FUNC}()",
@@ -677,7 +617,7 @@ class Base(object):
             logger.debug(f"Dropping trigger on table: {schema}.{table}")
             for name in ("notify", "truncate"):
                 query = (
-                    f"DROP TRIGGER IF EXISTS {table}_{name} ON "
+                    f'DROP TRIGGER IF EXISTS "{table}_{name}" ON '
                     f'"{schema}"."{table}"'
                 )
                 self.execute(query)
@@ -802,12 +742,13 @@ class Base(object):
         payload.update(match.groupdict())
         span = match.span()
         # trailing space is deliberate
-        suffix = f"{row[span[1]:]} "
+        suffix: str = f"{row[span[1]:]} "
+        tg_op: str = payload["tg_op"]
 
         if "old-key" and "new-tuple" in suffix:
             # this can only be an UPDATE operation
-            if payload["tg_op"] != UPDATE:
-                msg = f"Unknown {payload['tg_op']} operation for row: {row}"
+            if tg_op != UPDATE:
+                msg = f"Unknown {tg_op} operation for row: {row}"
                 raise LogicalSlotParseError(msg)
 
             i = suffix.index("old-key:")
@@ -824,9 +765,9 @@ class Base(object):
                     payload["new"][key] = value
         else:
             # this can be an INSERT, DELETE, UPDATE or TRUNCATE operation
-            if payload["tg_op"] not in TG_OP:
-                msg = f"Unknown {payload['tg_op']} operation for row: {row}"
-                raise LogicalSlotParseError(msg)
+            if tg_op not in TG_OP:
+                message = f"Unknown {tg_op} operation for row: {row}"
+                raise LogicalSlotParseError(message)
 
             for key, value in _parse_logical_slot(suffix):
                 payload["new"][key] = value
@@ -875,8 +816,12 @@ class Base(object):
             raise
         return rows
 
-    def fetchmany(self, statement, chunk_size=None):
-        chunk_size = chunk_size or QUERY_CHUNK_SIZE
+    def fetchmany(
+        self,
+        statement: sa.sql.selectable.Select,
+        chunk_size: Optional[int] = None,
+    ):
+        chunk_size: int = chunk_size or QUERY_CHUNK_SIZE
         with self.__engine.connect() as conn:
             result = conn.execution_options(stream_results=True).execute(
                 statement.select()
@@ -917,9 +862,9 @@ def subtransactions(session):
     return ControlledExecution(session)
 
 
-def _get_foreign_keys(model_a, model_b):
+def _get_foreign_keys(model_a, model_b) -> dict:
 
-    foreign_keys = defaultdict(list)
+    foreign_keys: dict = defaultdict(list)
 
     if model_a.foreign_keys:
 
@@ -940,8 +885,7 @@ def _get_foreign_keys(model_a, model_b):
     if not foreign_keys:
         raise ForeignKeyError(
             f"No foreign key relationship between "
-            f'"{model_a.original}" and '
-            f'"{model_b.original}"'
+            f'"{model_a.original}" and "{model_b.original}"'
         )
 
     return foreign_keys
@@ -1091,7 +1035,7 @@ def compiled_query(
     """Compile an SQLAlchemy query with an optional label."""
     query: str = str(
         query.compile(
-            dialect=postgresql.dialect(),
+            dialect=sa.dialects.postgresql.dialect(),
             compile_kwargs={"literal_binds": literal_binds},
         )
     )
