@@ -16,6 +16,8 @@ from .constants import (
 )
 from .node import Node, traverse_post_order
 from .settings import (
+    ELASTICSEARCH_API_KEY,
+    ELASTICSEARCH_API_KEY_ID,
     ELASTICSEARCH_AWS_HOSTED,
     ELASTICSEARCH_AWS_REGION,
     ELASTICSEARCH_CA_CERTS,
@@ -36,7 +38,7 @@ from .settings import (
     ELASTICSEARCH_USE_SSL,
     ELASTICSEARCH_VERIFY_CERTS,
 )
-from .utils import get_elasticsearch_url
+from .urls import get_elasticsearch_url
 
 logger = logging.getLogger(__name__)
 
@@ -53,12 +55,21 @@ class ElasticHelper(object):
         """
         url: str = get_elasticsearch_url()
         self.__es: Elasticsearch = get_elasticsearch_client(url)
+        self.opensearch: bool = False
         try:
             self.major_version: int = int(
                 self.__es.info()["version"]["number"].split(".")[0]
             )
+            self.opensearch = (
+                self.__es.info()["tagline"] != "You Know, for Search"
+            )
         except (IndexError, KeyError, ValueError):
             self.major_version: int = 0
+        self.doc_count: int = 0
+
+    def close(self) -> None:
+        """Close transport connection."""
+        self.__es.transport.close()
 
     def teardown(self, index: str) -> None:
         """
@@ -87,6 +98,41 @@ class ElasticHelper(object):
         max_backoff: Optional[int] = None,
         raise_on_exception: Optional[bool] = None,
         raise_on_error: Optional[bool] = None,
+    ) -> None:
+        """Pull sync data from generator to Elasticsearch."""
+        try:
+            self._bulk(
+                index,
+                docs,
+                chunk_size=chunk_size,
+                max_chunk_bytes=max_chunk_bytes,
+                queue_size=queue_size,
+                thread_count=thread_count,
+                refresh=refresh,
+                max_retries=max_retries,
+                initial_backoff=initial_backoff,
+                max_backoff=max_backoff,
+                raise_on_exception=raise_on_exception,
+                raise_on_error=raise_on_error,
+            )
+        except Exception as e:
+            logger.exception(f"Exception {e}")
+            raise
+
+    def _bulk(
+        self,
+        index: str,
+        docs: Generator,
+        chunk_size: Optional[int] = None,
+        max_chunk_bytes: Optional[int] = None,
+        queue_size: Optional[int] = None,
+        thread_count: Optional[int] = None,
+        refresh: bool = False,
+        max_retries: Optional[int] = None,
+        initial_backoff: Optional[int] = None,
+        max_backoff: Optional[int] = None,
+        raise_on_exception: Optional[bool] = None,
+        raise_on_error: Optional[bool] = None,
     ):
         """Bulk index, update, delete docs to Elasticsearch."""
         chunk_size: int = chunk_size or ELASTICSEARCH_CHUNK_SIZE
@@ -117,7 +163,7 @@ class ElasticHelper(object):
                 raise_on_exception=raise_on_exception,
                 raise_on_error=raise_on_error,
             ):
-                pass
+                self.doc_count += 1
         else:
             # parallel bulk consumes more memory and is also more likely
             # to result in 429 errors.
@@ -132,13 +178,13 @@ class ElasticHelper(object):
                 raise_on_exception=raise_on_exception,
                 raise_on_error=raise_on_error,
             ):
-                pass
+                self.doc_count += 1
 
     def refresh(self, indices: List[str]) -> None:
         """Refresh the Elasticsearch index."""
         self.__es.indices.refresh(index=indices)
 
-    def _search(self, index: str, table: str, fields: dict = None):
+    def _search(self, index: str, table: str, fields: Optional[dict] = None):
         """
         Search private area for matching docs in Elasticsearch.
 
@@ -181,6 +227,7 @@ class ElasticHelper(object):
         index: str,
         node: None,
         setting: Optional[dict] = None,
+        mapping: Optional[dict] = None,
         routing: Optional[str] = None,
     ) -> None:
         """Create Elasticsearch setting and mapping if required."""
@@ -191,15 +238,14 @@ class ElasticHelper(object):
             if setting:
                 body.update(**{"settings": {"index": setting}})
 
-            mapping: dict = self._build_mapping(node, routing)
             if mapping:
-                body.update(**mapping)
-
+                body.update(**{"mappings": {"properties": mapping}})
+            else:
+                mapping: dict = self._build_mapping(node, routing)
+                if mapping:
+                    body.update(**mapping)
             try:
-                response = self.__es.indices.create(
-                    index=index,
-                    body=body,
-                )
+                response = self.__es.indices.create(index=index, body=body)
             except Exception:
                 raise
 
@@ -213,13 +259,12 @@ class ElasticHelper(object):
                 f"created setting: {self.__es.indices.get_settings(index)}"
             )
 
-    def _build_mapping(self, root: Node, routing: str):
+    def _build_mapping(self, root: Node, routing: str) -> Optional[dict]:
         """Get the Elasticsearch mapping from the schema transform."""
-
         for node in traverse_post_order(root):
 
-            rename = node.transform.get("rename", {})
-            mapping = node.transform.get("mapping", {})
+            rename: dict = node.transform.get("rename", {})
+            mapping: dict = node.transform.get("mapping", {})
 
             for key, value in mapping.items():
                 column: str = rename.get(key, key)
@@ -256,14 +301,13 @@ class ElasticHelper(object):
             root._mapping["_routing"] = {"required": True}
 
         if root._mapping:
-            if self.major_version < 7:
+            if self.major_version < 7 and not self.opensearch:
                 root._mapping = {"_doc": root._mapping}
 
             return dict(mappings=root._mapping)
 
 
 def get_elasticsearch_client(url: str) -> Elasticsearch:
-
     if ELASTICSEARCH_AWS_HOSTED:
         credentials = boto3.Session().get_credentials()
         return Elasticsearch(
@@ -289,4 +333,5 @@ def get_elasticsearch_client(url: str) -> Elasticsearch:
             ca_certs=ELASTICSEARCH_CA_CERTS,
             client_cert=ELASTICSEARCH_CLIENT_CERT,
             client_key=ELASTICSEARCH_CLIENT_KEY,
+            api_key=(ELASTICSEARCH_API_KEY_ID, ELASTICSEARCH_API_KEY),
         )
