@@ -13,13 +13,47 @@ class QueryBuilder(object):
     """Query builder."""
 
     def __init__(self, base: Base, verbose: bool = False):
-        """
-        Query builder constructor.
-        """
+        """Query builder constructor."""
         self.base: Base = base
         self.verbose: bool = verbose
         self.isouter: bool = True
 
+    def _json_build_object(
+        self, columns: list
+    ) -> sa.sql.elements.BinaryExpression:
+        """
+        Tries to get aroud the limitation of JSON_BUILD_OBJECT which
+        has a default limit of 100 args.
+        This results in the error: 'cannot pass more than 100 arguments
+        to a function'
+
+        with the 100 arguments limit this implies we can only select 50 columns
+        at a time.
+        """
+        i: int = 0
+        chunk_size: int = 100
+        expression: sa.sql.elements.BinaryExpression = None
+        while i < len(columns):
+            chunk = columns[i : i + chunk_size]
+            if i == 0:
+                expression = sa.cast(
+                    sa.func.JSON_BUILD_OBJECT(*chunk),
+                    sa.dialects.postgresql.JSONB,
+                )
+            else:
+                expression = expression.concat(
+                    sa.cast(
+                        sa.func.JSON_BUILD_OBJECT(*chunk),
+                        sa.dialects.postgresql.JSONB,
+                    )
+                )
+            i += chunk_size
+
+        if expression is None:
+            raise RuntimeError("invalid expression")
+
+        return expression
+
     def _get_foreign_keys(self, node_a: Node, node_b: Node) -> Dict:
         if (
             node_a.relationship.through_tables
@@ -27,15 +61,15 @@ class QueryBuilder(object):
         ):
 
             if node_a.relationship.through_tables:
-                through_tables = node_a.relationship.through_tables[0]
+                through_table: str = node_a.relationship.through_tables[0]
 
             if node_b.relationship.through_tables:
-                through_tables = node_b.relationship.through_tables[0]
+                through_table: str = node_b.relationship.through_tables[0]
                 node_a, node_b = node_b, node_a
 
             through: Node = node_from_table(
                 self.base,
-                through_tables,
+                through_table,
                 node_a.schema,
             )
             foreign_keys: dict = get_foreign_keys(node_a, through)
@@ -136,7 +170,7 @@ class QueryBuilder(object):
                     )
                 ]
             ),
-            sa.func.JSON_BUILD_OBJECT(*node.columns),
+            self._json_build_object(node.columns),
             *node.primary_keys,
         ]
         node._subquery = sa.select(columns)
@@ -159,7 +193,7 @@ class QueryBuilder(object):
                 child.parent.columns.extend(
                     [
                         child.label,
-                        getattr(child._subquery.c, child.label),
+                        child._subquery.c[child.label],
                     ]
                 )
 
@@ -183,14 +217,8 @@ class QueryBuilder(object):
 
                 for i in range(len(right_foreign_keys)):
                     onclause.append(
-                        getattr(
-                            child._subquery.c,
-                            left_foreign_keys[i],
-                        )
-                        == getattr(
-                            child.parent.model.c,
-                            right_foreign_keys[i],
-                        )
+                        child._subquery.c[left_foreign_keys[i]]
+                        == child.parent.model.c[right_foreign_keys[i]]
                     )
 
             else:
@@ -198,7 +226,7 @@ class QueryBuilder(object):
                 child.parent.columns.extend(
                     [
                         child.label,
-                        getattr(child._subquery.c, child.label),
+                        child._subquery.c[child.label],
                     ]
                 )
 
@@ -218,14 +246,8 @@ class QueryBuilder(object):
 
                 for i in range(len(left_foreign_keys)):
                     onclause.append(
-                        getattr(
-                            child._subquery.c,
-                            left_foreign_keys[i],
-                        )
-                        == getattr(
-                            child.parent.model.c,
-                            right_foreign_keys[i],
-                        )
+                        child._subquery.c[left_foreign_keys[i]]
+                        == child.parent.model.c[right_foreign_keys[i]]
                     )
 
             if self.from_obj is None:
@@ -243,10 +265,7 @@ class QueryBuilder(object):
                                     _column = node.model.c
                                 if hasattr(_column, column._orig_key):
                                     onclause.append(
-                                        getattr(
-                                            _column,
-                                            column._orig_key,
-                                        )
+                                        _column[column._orig_key]
                                         == column.value
                                     )
                     elif isinstance(
@@ -262,10 +281,7 @@ class QueryBuilder(object):
                                         _column = node.model.c
                                     if hasattr(_column, column._orig_key):
                                         onclause.append(
-                                            getattr(
-                                                _column,
-                                                column._orig_key,
-                                            )
+                                            _column[column._orig_key]
                                             == column.value
                                         )
                 if self.verbose:
@@ -307,12 +323,7 @@ class QueryBuilder(object):
             params.append(
                 sa.func.JSON_BUILD_OBJECT(
                     str(foreign_key_column),
-                    sa.func.JSON_BUILD_ARRAY(
-                        getattr(
-                            node.model.c,
-                            foreign_key_column,
-                        )
-                    ),
+                    sa.func.JSON_BUILD_ARRAY(node.model.c[foreign_key_column]),
                 )
             )
 
@@ -337,19 +348,16 @@ class QueryBuilder(object):
                     )
                 else:
                     columns.append(
-                        sa.func.JSON_BUILD_OBJECT(*node.columns).label("anon")
+                        self._json_build_object(node.columns).label("anon")
                     )
             elif node.relationship.type == ONE_TO_MANY:
                 columns.append(
-                    sa.func.JSON_BUILD_OBJECT(*node.columns).label("anon")
+                    self._json_build_object(node.columns).label("anon")
                 )
 
         columns.extend(
             [
-                getattr(
-                    node.model.c,
-                    foreign_key_column,
-                )
+                node.model.c[foreign_key_column]
                 for foreign_key_column in foreign_key_columns
             ]
         )
@@ -396,14 +404,8 @@ class QueryBuilder(object):
 
             for i in range(len(left_foreign_keys)):
                 onclause.append(
-                    getattr(
-                        child._subquery.c,
-                        left_foreign_keys[i],
-                    )
-                    == getattr(
-                        child.parent.model.c,
-                        right_foreign_keys[i],
-                    )
+                    child._subquery.c[left_foreign_keys[i]]
+                    == child.parent.model.c[right_foreign_keys[i]]
                 )
 
             if from_obj is None:
@@ -425,10 +427,7 @@ class QueryBuilder(object):
                                     column._orig_key,
                                 ):
                                     onclause.append(
-                                        getattr(
-                                            _column,
-                                            column._orig_key,
-                                        )
+                                        _column[column._orig_key]
                                         == column.value
                                     )
                     elif isinstance(
@@ -444,10 +443,7 @@ class QueryBuilder(object):
                                         _column = node.model.c
                                     if hasattr(_column, column._orig_key):
                                         onclause.append(
-                                            getattr(
-                                                _column,
-                                                column._orig_key,
-                                            )
+                                            _column[column._orig_key]
                                             == column.value
                                         )
 
@@ -474,12 +470,7 @@ class QueryBuilder(object):
             params.append(
                 sa.func.JSON_BUILD_OBJECT(
                     str(primary_key),
-                    sa.func.JSON_BUILD_ARRAY(
-                        getattr(
-                            through.model.c,
-                            primary_key,
-                        )
-                    ),
+                    sa.func.JSON_BUILD_ARRAY(through.model.c[primary_key]),
                 )
             )
 
@@ -515,7 +506,7 @@ class QueryBuilder(object):
         foreign_keys = get_foreign_keys(node.parent, through)
 
         for column in foreign_keys[through.name]:
-            columns.append(getattr(through.model.c, column))
+            columns.append(through.model.c[column])
 
         inner_subquery = sa.select(columns)
 
@@ -525,14 +516,8 @@ class QueryBuilder(object):
         onclause = []
         for i in range(len(left_foreign_keys)):
             onclause.append(
-                getattr(
-                    outer_subquery.c,
-                    left_foreign_keys[i],
-                )
-                == getattr(
-                    through.model.c,
-                    right_foreign_keys[i],
-                )
+                outer_subquery.c[left_foreign_keys[i]]
+                == through.model.c[right_foreign_keys[i]]
             )
 
         if node._filters:
@@ -553,13 +538,7 @@ class QueryBuilder(object):
             subquery = subquery.where(sa.and_(*node._filters))
 
         subquery = subquery.group_by(
-            *[
-                getattr(
-                    through.model.c,
-                    column,
-                )
-                for column in foreign_keys[through.name]
-            ]
+            *[through.model.c[column] for column in foreign_keys[through.name]]
         )
 
         if self.verbose:
@@ -584,14 +563,8 @@ class QueryBuilder(object):
 
             for i in range(len(foreign_key_columns)):
                 onclause.append(
-                    getattr(
-                        child._subquery.c,
-                        foreign_key_columns[i],
-                    )
-                    == getattr(
-                        node.model.c,
-                        foreign_keys[node.name][i],
-                    )
+                    child._subquery.c[foreign_key_columns[i]]
+                    == node.model.c[foreign_keys[node.name][i]]
                 )
 
             if from_obj is None:
@@ -609,10 +582,7 @@ class QueryBuilder(object):
                                     _column = node.model.c
                                 if hasattr(_column, column._orig_key):
                                     onclause.append(
-                                        getattr(
-                                            _column,
-                                            column._orig_key,
-                                        )
+                                        _column[column._orig_key]
                                         == column.value
                                     )
                     elif isinstance(
@@ -628,10 +598,7 @@ class QueryBuilder(object):
                                         _column = node.model.c
                                     if hasattr(_column, column._orig_key):
                                         onclause.append(
-                                            getattr(
-                                                _column,
-                                                column._orig_key,
-                                            )
+                                            _column[column._orig_key]
                                             == column.value
                                         )
 
@@ -659,10 +626,7 @@ class QueryBuilder(object):
                     [
                         str(primary_key.name),
                         sa.func.JSON_BUILD_ARRAY(
-                            getattr(
-                                node.model.c,
-                                primary_key.name,
-                            )
+                            node.model.c[primary_key.name]
                         ),
                     ]
                 )
@@ -671,20 +635,15 @@ class QueryBuilder(object):
                 params.extend(
                     [
                         str(primary_key.name),
-                        getattr(
-                            node.model.c,
-                            primary_key.name,
-                        ),
+                        node.model.c[primary_key.name],
                     ]
                 )
 
         if node.relationship.type == ONE_TO_ONE:
-            _keys = self._get_child_keys(
-                node, sa.func.JSON_BUILD_OBJECT(*params)
-            )
+            _keys = self._get_child_keys(node, self._json_build_object(params))
         elif node.relationship.type == ONE_TO_MANY:
             _keys = self._get_child_keys(
-                node, sa.func.JSON_AGG(sa.func.JSON_BUILD_OBJECT(*params))
+                node, sa.func.JSON_AGG(self._json_build_object(params))
             )
 
         columns: List = [_keys]
@@ -692,40 +651,27 @@ class QueryBuilder(object):
         if node.relationship.variant == SCALAR:
             # TODO: Raise exception here if the number of columns > 1
             if node.relationship.type == ONE_TO_ONE:
-                columns.append(
-                    getattr(
-                        node.model.c,
-                        node.columns[0],
-                    ).label(node.label)
-                )
+                columns.append(node.model.c[node.columns[0]].label(node.label))
             elif node.relationship.type == ONE_TO_MANY:
                 columns.append(
-                    sa.func.JSON_AGG(
-                        getattr(
-                            node.model.c,
-                            node.columns[0],
-                        )
-                    ).label(node.label)
+                    sa.func.JSON_AGG(node.model.c[node.columns[0]]).label(
+                        node.label
+                    )
                 )
         elif node.relationship.variant == OBJECT:
             if node.relationship.type == ONE_TO_ONE:
                 columns.append(
-                    sa.func.JSON_BUILD_OBJECT(*node.columns).label(node.label)
+                    self._json_build_object(node.columns).label(node.label)
                 )
             elif node.relationship.type == ONE_TO_MANY:
                 columns.append(
                     sa.func.JSON_AGG(
-                        sa.func.JSON_BUILD_OBJECT(*node.columns)
+                        self._json_build_object(node.columns)
                     ).label(node.label)
                 )
 
         for column in foreign_key_columns:
-            columns.append(
-                getattr(
-                    node.model.c,
-                    column,
-                )
-            )
+            columns.append(node.model.c[column])
 
         node._subquery = sa.select(columns)
 
@@ -737,13 +683,7 @@ class QueryBuilder(object):
 
         if node.relationship.type == ONE_TO_MANY:
             node._subquery = node._subquery.group_by(
-                *[
-                    getattr(
-                        node.model.c,
-                        key,
-                    )
-                    for key in foreign_key_columns
-                ]
+                *[node.model.c[key] for key in foreign_key_columns]
             )
 
         node._subquery = node._subquery.alias()
