@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-"""Main module."""
+"""Sync module."""
 import json
 import logging
 import os
@@ -9,18 +9,17 @@ import re
 import select
 import sys
 import time
-import uuid
 from collections import defaultdict
-from datetime import datetime, timedelta
 from typing import AnyStr, Generator, List, Optional, Set
 
 import click
-import psycopg2
 import sqlalchemy as sa
+from psycopg2 import OperationalError
+from psycopg2.extensions import ISOLATION_LEVEL_AUTOCOMMIT
 from sqlalchemy.sql import Values
 
 from . import __version__
-from .base import Base, compiled_query, get_foreign_keys
+from .base import Base, compiled_query, get_foreign_keys, TupleIdentifierType
 from .constants import (
     DELETE,
     INSERT,
@@ -41,6 +40,7 @@ from .exc import (
 from .node import (
     get_node,
     Node,
+    node_from_table,
     traverse_breadth_first,
     traverse_post_order,
     Tree,
@@ -50,8 +50,10 @@ from .querybuilder import QueryBuilder
 from .redisqueue import RedisQueue
 from .settings import (
     CHECKPOINT_PATH,
+    LOG_INTERVAL,
     POLL_TIMEOUT,
     REDIS_POLL_INTERVAL,
+    REDIS_WRITE_CHUNK_SIZE,
     REPLICATION_SLOT_CLEANUP_INTERVAL,
 )
 from .transform import get_private_keys, transform
@@ -77,6 +79,7 @@ class Sync(Base):
         self.plugins: list = document.get("plugins", [])
         self.nodes: dict = document.get("nodes", {})
         self.setting: dict = document.get("setting")
+        self.mapping: dict = document.get("mapping")
         self.routing: str = document.get("routing")
         super().__init__(
             document.get("database", self.index), verbose=verbose, **kwargs
@@ -93,14 +96,13 @@ class Sync(Base):
         )
         self.redis: RedisQueue = RedisQueue(self.__name)
         self.tree: Tree = Tree(self)
-        self._last_truncate_timestamp: datetime = datetime.now()
         if validate:
             self.validate(repl_slots=repl_slots)
             self.create_setting()
         self.query_builder: QueryBuilder = QueryBuilder(
             self, verbose=self.verbose
         )
-        self.count: dict = dict(db=0, redis=0, elastic=0)
+        self.count: dict = dict(xlog=0, db=0, redis=0)
 
     def validate(self, repl_slots: Optional[bool] = True) -> None:
         """Perform all validation right away."""
@@ -175,6 +177,61 @@ class Sync(Base):
         for node in traverse_breadth_first(root):
             pass
 
+    def analyze(self) -> None:
+        root: Node = self.tree.build(self.nodes)
+        for node in traverse_breadth_first(root):
+
+            if node.is_root:
+                continue
+
+            primary_keys: list = [
+                str(primary_key.name) for primary_key in node.primary_keys
+            ]
+
+            if node.relationship.through_tables:
+                through_table: str = node.relationship.through_tables[0]
+                through: Node = node_from_table(
+                    self,
+                    through_table,
+                    node.schema,
+                )
+                foreign_keys = get_foreign_keys(
+                    node.parent,
+                    through,
+                )
+            else:
+                foreign_keys = get_foreign_keys(
+                    node.parent,
+                    node,
+                )
+
+            for index in self.indices(node.table):
+                columns: list = foreign_keys.get(node.name, [])
+                if set(columns).issubset(index.get("column_names", [])) or set(
+                    columns
+                ).issubset(primary_keys):
+                    sys.stdout.write(
+                        f'Found index "{index.get("name")}" for table '
+                        f'"{node.table}" for columns: {columns}: OK \n'
+                    )
+                    break
+            else:
+                columns: list = foreign_keys.get(node.name, [])
+                sys.stdout.write(
+                    f'Missing index on table "{node.table}" for columns: '
+                    f"{columns}\n"
+                )
+                command: str = (
+                    f'CREATE INDEX idx_{node.table}_{"_".join(columns)} ON '
+                    f'{node.table} ({", ".join(columns)})'
+                )
+                sys.stdout.write(
+                    f'Create one with: "\033[4m{command}\033[0m"\n'
+                )
+                sys.stdout.write("-" * 80)
+                sys.stdout.write("\n")
+                sys.stdout.flush()
+
     def create_setting(self) -> None:
         """Create Elasticsearch setting and mapping if required."""
         root: Node = self.tree.build(self.nodes)
@@ -182,6 +239,7 @@ class Sync(Base):
             self.index,
             root,
             setting=self.setting,
+            mapping=self.mapping,
             routing=self.routing,
         )
 
@@ -213,18 +271,19 @@ class Sync(Base):
                     user_defined_fkey_tables.setdefault(node.table, set([]))
                     user_defined_fkey_tables[node.table] |= set(columns)
             if tables:
-                self.create_triggers(schema, tables=tables)
                 self.create_view(schema, tables, user_defined_fkey_tables)
+                self.create_triggers(schema, tables=tables)
         self.create_replication_slot(self.__name)
 
     def teardown(self, drop_view: bool = True) -> None:
         """Drop the database triggers and replication slot."""
-
         try:
             os.unlink(self._checkpoint_file)
         except OSError:
             pass
 
+        self.redis._delete()
+
         for schema in self.schemas:
             tables: Set = set([])
             root: Node = self.tree.build(self.nodes)
@@ -316,10 +375,10 @@ class Sync(Base):
                     payload["tg_op"] != payload2["tg_op"]
                     or payload["table"] != payload2["table"]
                 ):
-                    self.sync(self._payloads(payloads))
+                    self.es.bulk(self.index, self._payloads(payloads))
                     payloads: list = []
             elif j == len(_rows):
-                self.sync(self._payloads(payloads))
+                self.es.bulk(self.index, self._payloads(payloads))
                 payloads: list = []
 
         if rows:
@@ -329,6 +388,7 @@ class Sync(Base):
                 txmax=txmax,
                 upto_nchanges=len(rows),
             )
+            self.count["xlog"] += len(rows)
 
     def _payload_data(self, payload: dict) -> dict:
         """Extract the payload data from the payload."""
@@ -338,7 +398,7 @@ class Sync(Base):
                 payload_data = payload.get("old")
         return payload_data
 
-    def _insert(
+    def _insert_op(
         self, node: Node, root: Node, filters: dict, payloads: dict
     ) -> None:
 
@@ -401,7 +461,7 @@ class Sync(Base):
 
         return filters
 
-    def _update(
+    def _update_op(
         self,
         node: Node,
         root: Node,
@@ -451,7 +511,7 @@ class Sync(Base):
                     }
                     if self.routing:
                         doc["_routing"] = old_values[self.routing]
-                    if self.es.major_version < 7:
+                    if self.es.major_version < 7 and not self.es.opensearch:
                         doc["_type"] = "_doc"
                     docs.append(doc)
 
@@ -533,7 +593,7 @@ class Sync(Base):
 
         return filters
 
-    def _delete(
+    def _delete_op(
         self, node: Node, root: Node, filters: dict, payloads: dict
     ) -> None:
 
@@ -553,7 +613,7 @@ class Sync(Base):
                 }
                 if self.routing:
                     doc["_routing"] = payload_data[self.routing]
-                if self.es.major_version < 7:
+                if self.es.major_version < 7 and not self.es.opensearch:
                     doc["_type"] = "_doc"
                 docs.append(doc)
             if docs:
@@ -591,7 +651,7 @@ class Sync(Base):
 
         return filters
 
-    def _truncate(self, node: Node, root: Node, filters: dict) -> dict:
+    def _truncate_op(self, node: Node, root: Node, filters: dict) -> dict:
 
         if node.table == root.table:
 
@@ -602,7 +662,7 @@ class Sync(Base):
                     "_index": self.index,
                     "_op_type": "delete",
                 }
-                if self.es.major_version < 7:
+                if self.es.major_version < 7 and not self.es.opensearch:
                     doc["_type"] = "_doc"
                 docs.append(doc)
             if docs:
@@ -691,7 +751,7 @@ class Sync(Base):
 
         if tg_op == INSERT:
 
-            filters = self._insert(
+            filters = self._insert_op(
                 node,
                 root,
                 filters,
@@ -700,7 +760,7 @@ class Sync(Base):
 
         if tg_op == UPDATE:
 
-            filters = self._update(
+            filters = self._update_op(
                 node,
                 root,
                 filters,
@@ -710,7 +770,7 @@ class Sync(Base):
 
         if tg_op == DELETE:
 
-            filters = self._delete(
+            filters = self._delete_op(
                 node,
                 root,
                 filters,
@@ -719,13 +779,13 @@ class Sync(Base):
 
         if tg_op == TRUNCATE:
 
-            filters = self._truncate(node, root, filters)
+            filters = self._truncate_op(node, root, filters)
 
         # If there are no filters, then don't execute the sync query
         # otherwise we would end up performing a full query
         # and sync the entire db!
         if any(filters.values()):
-            yield from self._sync(filters=filters, extra=extra)
+            yield from self.sync(filters=filters, extra=extra)
 
     def _build_filters(self, filters: List[dict], node: Node) -> None:
         """
@@ -746,47 +806,20 @@ class Sync(Base):
             for _filter in filters.get(node.table):
                 where: list = []
                 for key, value in _filter.items():
-                    where.append(getattr(node.model.c, key) == value)
+                    where.append(node.model.c[key] == value)
                     keys.add(key)
                     values.add(value)
                 _filters.append(sa.and_(*where))
 
-            if len(keys) == 1:
-                # If we have the same key then the node does not have a
-                # compound primary key
-                column: list = list(keys)[0]
-                _values = []
-                for value in values:
-                    if (
-                        str(getattr(node.model.c, column).type).lower()
-                        == "uuid"
-                    ):
-                        _values.append((uuid.UUID(value),))
-                    else:
-                        _values.append(
-                            (
-                                getattr(node.model.c, column).type.python_type(
-                                    value
-                                ),
-                            )
-                        )
+            node._filters.append(sa.or_(*_filters))
 
-                node._filters.append(
-                    getattr(node.model.c, column).in_(
-                        sa.select(
-                            Values(sa.column(column)).data(_values).alias("x")
-                        )
-                    )
-                )
-            else:
-                node._filters.append(sa.or_(*_filters))
-
-    def _sync(
+    def sync(
         self,
         filters: Optional[dict] = None,
         txmin: Optional[int] = None,
         txmax: Optional[int] = None,
         extra: Optional[dict] = None,
+        ctid: Optional[int] = None,
     ) -> Generator:
         if filters is None:
             filters: dict = {}
@@ -800,6 +833,41 @@ class Sync(Base):
             self._build_filters(filters, node)
 
             if node.is_root:
+
+                if ctid is not None:
+                    subquery = []
+                    for page, rows in ctid.items():
+                        subquery.append(
+                            sa.select(
+                                [
+                                    sa.cast(
+                                        sa.literal_column(f"'({page},'")
+                                        .concat(sa.column("s"))
+                                        .concat(")"),
+                                        TupleIdentifierType,
+                                    )
+                                ]
+                            ).select_from(
+                                Values(
+                                    sa.column("s"),
+                                )
+                                .data([(row,) for row in rows])
+                                .alias("s")
+                            )
+                        )
+                    if subquery:
+                        node._filters.append(
+                            sa.or_(
+                                *[
+                                    node.model.c.ctid
+                                    == sa.any_(
+                                        sa.func.ARRAY(q.scalar_subquery())
+                                    )
+                                    for q in subquery
+                                ]
+                            )
+                        )
+
                 if txmin:
                     node._filters.append(
                         sa.cast(
@@ -832,7 +900,7 @@ class Sync(Base):
         if self.verbose:
             compiled_query(node._subquery, "Query")
 
-        count = self.fetchcount(node._subquery)
+        count: int = self.fetchcount(node._subquery)
 
         with click.progressbar(
             length=count,
@@ -873,7 +941,7 @@ class Sync(Base):
                 if self.routing:
                     doc["_routing"] = row[self.routing]
 
-                if self.es.major_version < 7:
+                if self.es.major_version < 7 and not self.es.opensearch:
                     doc["_type"] = "_doc"
 
                 if self._plugins:
@@ -884,16 +952,6 @@ class Sync(Base):
 
                 yield doc
 
-    def sync(self, docs: Generator) -> None:
-        """
-        Pull sync data from generator to Elasticsearch.
-        """
-        try:
-            self.es.bulk(self.index, docs)
-        except Exception as e:
-            logger.exception(f"Exception {e}")
-            raise
-
     @property
     def checkpoint(self) -> int:
         """Save the current txid as the checkpoint."""
@@ -929,42 +987,38 @@ class Sync(Base):
         Receive a notification message from the channel we are listening on
         """
         conn = self.engine.connect().connection
-        conn.set_isolation_level(
-            psycopg2.extensions.ISOLATION_LEVEL_AUTOCOMMIT
-        )
+        conn.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
         cursor = conn.cursor()
         channel: str = self.database
         cursor.execute(f'LISTEN "{channel}"')
         logger.debug(f'Listening for notifications on channel "{channel}"')
+        items: list = []
 
-        i: int = 0
         while True:
             # NB: consider reducing POLL_TIMEOUT to increase throughout
             if select.select([conn], [], [], POLL_TIMEOUT) == ([], [], []):
-                if i % 10 == 0:
-                    sys.stdout.write(
-                        f"Syncing {channel} "
-                        f"db: [{self.count['db']:,}] => "
-                        f"redis: [{self.count['redis']:,}] => "
-                        f"elastic: [{self.count['elastic']:,}]...\n"
-                    )
-                    sys.stdout.flush()
-                i += 1
+                # Catch any hanging items from the last poll
+                if items:
+                    self.redis.bulk_push(items)
+                    items = []
                 continue
 
             try:
                 conn.poll()
-            except psycopg2.OperationalError as e:
+            except OperationalError as e:
                 logger.fatal(f"OperationalError: {e}")
                 os._exit(-1)
 
             while conn.notifies:
+                if len(items) >= REDIS_WRITE_CHUNK_SIZE:
+                    self.redis.bulk_push(items)
+                    items = []
                 notification: AnyStr = conn.notifies.pop(0)
-                payload = json.loads(notification.payload)
-                self.redis.push(payload)
-                logger.debug(f"on_notify: {payload}")
-                self.count["db"] += 1
-            i = 0
+                if notification.channel == channel:
+                    payload = json.loads(notification.payload)
+                    items.append(payload)
+                    logger.debug(f"on_notify: {payload}")
+                    self.count["db"] += 1
 
     def on_publish(self, payloads: list) -> None:
         """
@@ -987,7 +1041,7 @@ class Sync(Base):
                 _payloads[payload["table"]].append(payload)
 
             for _payload in _payloads.values():
-                self.sync(self._payloads(_payload))
+                self.es.bulk(self.index, self._payloads(_payload))
 
         else:
 
@@ -1001,14 +1055,12 @@ class Sync(Base):
                         payload["tg_op"] != payload2["tg_op"]
                         or payload["table"] != payload2["table"]
                     ):
-                        self.sync(self._payloads(_payloads))
+                        self.es.bulk(self.index, self._payloads(_payloads))
                         _payloads = []
                 elif j == len(payloads):
-                    self.sync(self._payloads(_payloads))
+                    self.es.bulk(self.index, self._payloads(_payloads))
                     _payloads: list = []
 
-        self.count["elastic"] += len(payloads)
-
         txids: Set = set(map(lambda x: x["xmin"], payloads))
         # for truncate, tg_op txids is None so skip setting the checkpoint
         if txids != set([None]):
@@ -1018,9 +1070,9 @@ class Sync(Base):
         """Pull data from db."""
         txmin: int = self.checkpoint
         txmax: int = self.txid_current
-        logger.debug(f"pull txmin: {txmin} txmax: {txmax}")
+        logger.debug(f"pull txmin: {txmin} - txmax: {txmax}")
         # forward pass sync
-        self.sync(self._sync(txmin=txmin, txmax=txmax))
+        self.es.bulk(self.index, self.sync(txmin=txmin, txmax=txmax))
         self.checkpoint: int = txmax or self.txid_current
         # now sync up to txmax to capture everything we may have missed
         self.logical_slot_changes(txmin=txmin, txmax=txmax)
@@ -1030,15 +1082,24 @@ class Sync(Base):
     def truncate_slots(self) -> None:
         """Truncate the logical replication slot."""
         while True:
-            if self._truncate and (
-                datetime.now()
-                >= self._last_truncate_timestamp
-                + timedelta(seconds=REPLICATION_SLOT_CLEANUP_INTERVAL)
-            ):
+            if self._truncate:
                 logger.debug(f"Truncating replication slot: {self.__name}")
                 self.logical_slot_get_changes(self.__name, upto_nchanges=None)
-                self._last_truncate_timestamp = datetime.now()
-            time.sleep(0.1)
+            time.sleep(REPLICATION_SLOT_CLEANUP_INTERVAL)
+
+    @threaded
+    def status(self):
+        while True:
+            sys.stdout.write(
+                f"Syncing {self.database} "
+                f"Xlog: [{self.count['xlog']:,}] => "
+                f"Db: [{self.count['db']:,}] => "
+                f"Redis: [total = {self.count['redis']:,} "
+                f"pending = {self.redis.qsize:,}] => "
+                f"Elastic: [{self.es.doc_count:,}] ...\n"
+            )
+            sys.stdout.flush()
+            time.sleep(LOG_INTERVAL)
 
     def receive(self) -> None:
         """
@@ -1064,6 +1125,9 @@ class Sync(Base):
         # start a background worker thread to cleanup the replication slot
         self.truncate_slots()
 
+        # start a background worker thread to show status
+        self.status()
+
 
 @click.command()
 @click.option(
@@ -1110,6 +1174,13 @@ class Sync(Base):
     default=False,
     help="Show version info",
 )
+@click.option(
+    "--analyze",
+    "-a",
+    is_flag=True,
+    default=False,
+    help="Analyse database",
+)
 def main(
     config,
     daemon,
@@ -1121,10 +1192,9 @@ def main(
     user,
     verbose,
     version,
+    analyze,
 ):
-    """
-    main application syncer
-    """
+    """Main application syncer."""
     if version:
         sys.stdout.write(f"Version: {__version__}\n")
         return
@@ -1150,6 +1220,12 @@ def main(
 
     show_settings(config, **kwargs)
 
+    if analyze:
+        for document in json.load(open(config)):
+            sync = Sync(document, verbose=verbose, **kwargs)
+            sync.analyze()
+        return
+
     with Timer():
         for document in json.load(open(config)):
             sync = Sync(document, verbose=verbose, **kwargs)
