@@ -11,7 +11,7 @@ from sqlalchemy.schema import UniqueConstraint
 from pgsync.base import Base, create_database, drop_database
 from pgsync.constants import SCHEMA
 from pgsync.sync import Sync
-from pgsync.utils import get_postgres_url
+from pgsync.urls import get_postgres_url
 
 warnings.filterwarnings("error")
 
@@ -65,6 +65,7 @@ def sync():
     )
     _sync.engine.connect().close()
     _sync.engine.dispose()
+    _sync.session.close()
 
 
 def pytest_addoption(parser):
