@@ -7,7 +7,7 @@ import pytest
 from pgsync.base import subtransactions
 from pgsync.sync import Sync
 
-from .helpers.utils import assert_resync_empty, search, truncate_slots
+from .helpers.utils import assert_resync_empty, noop, search
 
 
 @pytest.mark.usefixtures("table_creator")
@@ -273,12 +273,14 @@ class TestNestedChildren(object):
 
         try:
             sync.es.teardown(index="testdb")
+            sync.es.close()
         except Exception:
             raise
 
         sync.redis._delete()
         session.connection().engine.connect().close()
         session.connection().engine.dispose()
+        sync.es.close()
 
     @pytest.fixture(scope="function")
     def nodes(self):
@@ -375,7 +377,7 @@ class TestNestedChildren(object):
     def test_sync(self, sync, nodes, data):
         """test regular sync produces the correct result."""
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         assert len(docs) == 3
         docs = sorted(docs, key=lambda k: k["_id"])
         assert docs == [
@@ -650,7 +652,7 @@ class TestNestedChildren(object):
 
         # 1. sync first to add the initial document
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.checkpoint = sync.txid_current
 
         session = sync.session
@@ -665,7 +667,7 @@ class TestNestedChildren(object):
 
         txmin = sync.checkpoint
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync(txmin=txmin)]
+        docs = [doc for doc in sync.sync(txmin=txmin)]
         assert len(docs) == 2
         docs = sorted(docs, key=lambda k: k["_id"])
         assert docs == [
@@ -764,6 +766,7 @@ class TestNestedChildren(object):
             },
         ]
         assert_resync_empty(sync, nodes)
+        sync.es.close()
 
     def test_update_root(self, data, nodes, book_cls):
         document = {
@@ -772,7 +775,7 @@ class TestNestedChildren(object):
         }
         # 1. sync first to add the initial document
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.checkpoint = sync.txid_current
 
         session = sync.session
@@ -786,7 +789,7 @@ class TestNestedChildren(object):
 
         txmin = sync.checkpoint
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync(txmin=txmin)]
+        docs = [doc for doc in sync.sync(txmin=txmin)]
 
         assert len(docs) == 1
         docs = sorted(docs, key=lambda k: k["_id"])
@@ -851,6 +854,7 @@ class TestNestedChildren(object):
             }
         ]
         assert_resync_empty(sync, nodes)
+        sync.es.close()
 
     def test_delete_root(
         self,
@@ -868,7 +872,7 @@ class TestNestedChildren(object):
         }
         # 1. sync first to add the initial document
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.checkpoint = sync.txid_current
 
         session = sync.session
@@ -915,14 +919,18 @@ class TestNestedChildren(object):
                 with mock.patch("pgsync.sync.Sync.pull", side_effect=pull):
                     with mock.patch(
                         "pgsync.sync.Sync.truncate_slots",
-                        side_effect=truncate_slots,
+                        side_effect=noop,
                     ):
-                        sync.receive()
-                        sync.es.refresh("testdb")
+                        with mock.patch(
+                            "pgsync.sync.Sync.status",
+                            side_effect=noop,
+                        ):
+                            sync.receive()
+                            sync.es.refresh("testdb")
 
         txmin = sync.checkpoint
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync(txmin=txmin)]
+        docs = [doc for doc in sync.sync(txmin=txmin)]
         assert len(docs) == 0
 
         docs = search(sync.es, "testdb")
@@ -1037,6 +1045,7 @@ class TestNestedChildren(object):
             },
         ]
         assert_resync_empty(sync, nodes)
+        sync.es.close()
 
     def test_insert_through_child_noop(self, sync, data):
         # insert a new through child with noop
@@ -1097,7 +1106,7 @@ class TestNestedChildren(object):
         with subtransactions(session):
             session.add(book_author)
 
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
@@ -1279,6 +1288,7 @@ class TestNestedChildren(object):
             },
         ]
         assert_resync_empty(sync, nodes)
+        sync.es.close()
 
     def test_update_through_child_op(
         self,
@@ -1292,7 +1302,6 @@ class TestNestedChildren(object):
         continent_cls,
     ):
         # update a new through child with op
-
         document = {
             "index": "testdb",
             "nodes": nodes,
@@ -1300,7 +1309,7 @@ class TestNestedChildren(object):
 
         # 1. sync first to add the initial document
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
 
         author = author_cls(
             id=5,
@@ -1328,7 +1337,7 @@ class TestNestedChildren(object):
                 .values(author_id=5)
             )
 
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
@@ -1497,6 +1506,7 @@ class TestNestedChildren(object):
             },
         ]
         assert_resync_empty(sync, nodes)
+        sync.es.close()
 
     def test_delete_through_child_op(self, sync, data, nodes, book_author_cls):
         # delete a new through child with op
@@ -1507,7 +1517,7 @@ class TestNestedChildren(object):
 
         # 1. sync first to add the initial document
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
 
         session = sync.session
 
@@ -1519,7 +1529,7 @@ class TestNestedChildren(object):
             )
             session.commit()
 
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
@@ -1656,6 +1666,7 @@ class TestNestedChildren(object):
             },
         ]
         assert_resync_empty(sync, nodes)
+        sync.es.close()
 
     def test_insert_nonthrough_child_noop(
         self,
@@ -1683,7 +1694,7 @@ class TestNestedChildren(object):
 
         # 1. sync first to add the initial document
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.checkpoint = sync.txid_current
         sync.es.refresh("testdb")
 
@@ -1697,10 +1708,11 @@ class TestNestedChildren(object):
 
         txmin = sync.checkpoint
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync(txmin=txmin)]
+        docs = [doc for doc in sync.sync(txmin=txmin)]
         assert len(docs) == 0
 
         assert_resync_empty(sync, nodes)
+        sync.es.close()
 
     def test_update_nonthrough_child_noop(self, data, nodes, shelf_cls):
         # update a new non-through child with noop
@@ -1711,7 +1723,7 @@ class TestNestedChildren(object):
 
         # 1. sync first to add the initial document
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.checkpoint = sync.txid_current
         sync.es.refresh("testdb")
 
@@ -1729,10 +1741,11 @@ class TestNestedChildren(object):
 
         txmin = sync.checkpoint
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync(txmin=txmin)]
+        docs = [doc for doc in sync.sync(txmin=txmin)]
         assert len(docs) == 0
 
         assert_resync_empty(sync, nodes)
+        sync.es.close()
 
     def test_delete_nonthrough_child_noop(self, data, nodes, shelf_cls):
         # delete a new non-through child with noop
@@ -1743,7 +1756,7 @@ class TestNestedChildren(object):
 
         # 1. sync first to add the initial document
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.checkpoint = sync.txid_current
         sync.checkpoint = sync.txid_current
         sync.es.refresh("testdb")
@@ -1762,10 +1775,11 @@ class TestNestedChildren(object):
 
         txmin = sync.checkpoint
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync(txmin=txmin)]
+        docs = [doc for doc in sync.sync(txmin=txmin)]
         assert len(docs) == 0
 
         assert_resync_empty(sync, nodes)
+        sync.es.close()
 
     def test_insert_nonthrough_child_op(self, sync, data):
         # insert a new non-through child with op
@@ -1894,7 +1908,7 @@ class TestNestedChildren(object):
         }
         # sync first to add the initial document
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.checkpoint = sync.txid_current
         session = sync.session
         sync.es.refresh("testdb")
@@ -1916,17 +1930,22 @@ class TestNestedChildren(object):
                 with mock.patch("pgsync.sync.Sync.pull", side_effect=pull):
                     with mock.patch(
                         "pgsync.sync.Sync.truncate_slots",
-                        side_effect=truncate_slots,
+                        side_effect=noop,
                     ):
-                        sync.receive()
-                        sync.es.refresh("testdb")
+                        with mock.patch(
+                            "pgsync.sync.Sync.status",
+                            side_effect=noop,
+                        ):
+                            sync.receive()
+                            sync.es.refresh("testdb")
 
         txmin = sync.checkpoint
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync(txmin=txmin)]
+        docs = [doc for doc in sync.sync(txmin=txmin)]
         assert docs == []
         docs = search(sync.es, "testdb")
         assert_resync_empty(sync, nodes)
+        sync.es.close()
 
     def test_insert_deep_nested_fk_nonthrough_child_op(
         self,
