@@ -11,7 +11,7 @@ from pgsync.exc import (
 )
 from pgsync.sync import Sync
 
-from .helpers.utils import assert_resync_empty, search, truncate_slots
+from .helpers.utils import assert_resync_empty, noop, search
 
 
 @pytest.mark.usefixtures("table_creator")
@@ -89,6 +89,7 @@ class TestRoot(object):
         sync.redis._delete()
         session.connection().engine.connect().close()
         session.connection().engine.dispose()
+        sync.es.close()
 
     def test_sync2(self, sync, data):
         """Test the sync with a root only node."""
@@ -96,7 +97,7 @@ class TestRoot(object):
         txmin = sync.checkpoint
         txmax = sync.txid_current
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync(txmin=txmin, txmax=txmax)]
+        docs = [doc for doc in sync.sync(txmin=txmin, txmax=txmax)]
         assert docs == [
             {
                 "_id": "abc",
@@ -146,7 +147,7 @@ class TestRoot(object):
             },
         }
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         assert docs == [
             {
                 "_id": "abc",
@@ -188,7 +189,7 @@ class TestRoot(object):
             "columns": ["isbn", "title", "description", "xmin"],
         }
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         assert sorted(docs[0]["_source"].keys()) == sorted(
             ["isbn", "title", "description", "xmin", "_meta"]
         )
@@ -198,7 +199,7 @@ class TestRoot(object):
         """Test the doc includes xmin column."""
         nodes = {"table": "book", "columns": ["isbn", "xmin"]}
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         assert "xmin" in docs[0]["_source"]
         assert_resync_empty(sync, nodes)
 
@@ -206,7 +207,7 @@ class TestRoot(object):
         """Test we include all columns when no columns are specified."""
         nodes = {"table": "book", "columns": []}
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         assert sorted(docs[0]["_source"].keys()) == sorted(
             [
                 "_meta",
@@ -220,7 +221,7 @@ class TestRoot(object):
 
         nodes = {"table": "book"}
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         assert sorted(docs[0]["_source"].keys()) == sorted(
             [
                 "_meta",
@@ -238,7 +239,7 @@ class TestRoot(object):
         nodes = {"table": "book", "columns": ["foo"]}
         sync.nodes = nodes
         with pytest.raises(ColumnNotFoundError) as excinfo:
-            [doc for doc in sync._sync()]
+            [doc for doc in sync.sync()]
         assert 'Column "foo" not present on table "book"' in str(excinfo.value)
 
     def test_primary_key_is_doc_id(self, sync, data):
@@ -246,7 +247,7 @@ class TestRoot(object):
         # TODO also repeat this test for composite primary key
         nodes = {"table": "book", "columns": ["title"]}
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         assert "abc" == docs[0]["_id"]
         assert "def" == docs[1]["_id"]
         assert "ghi" == docs[2]["_id"]
@@ -256,7 +257,7 @@ class TestRoot(object):
         """Test the private key is contained in the doc."""
         nodes = {"table": "book", "columns": ["isbn"]}
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         assert "_meta" in docs[0]["_source"]
         assert_resync_empty(sync, nodes)
 
@@ -264,7 +265,7 @@ class TestRoot(object):
         """Ensure the doc only selected columns and builtins."""
         nodes = {"table": "book", "columns": ["isbn", "xmin"]}
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         sources = {doc["_id"]: doc["_source"] for doc in docs}
 
         assert sorted(sources["abc"].keys()) == sorted(
@@ -281,7 +282,7 @@ class TestRoot(object):
             ],
         }
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         sources = {doc["_id"]: doc["_source"] for doc in docs}
 
         assert sorted(sources["abc"].keys()) == sorted(
@@ -297,7 +298,7 @@ class TestRoot(object):
         nodes = {"table": "book", "columns": ["copyright", "publisher_id"]}
         sync.nodes = nodes
 
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         sources = {doc["_id"]: doc["_source"] for doc in docs}
 
         assert sorted(sources["abc"].keys()) == sorted(
@@ -316,7 +317,7 @@ class TestRoot(object):
             "columns": ["isbn", "description", "copyright"],
         }
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         sources = {doc["_id"]: doc["_source"] for doc in docs}
 
         assert sources == {
@@ -345,7 +346,7 @@ class TestRoot(object):
         """Private keys should be included even if null."""
         nodes = {"table": "book", "columns": ["description"]}
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         sources = {doc["_id"]: doc["_source"] for doc in docs}
 
         assert sources["abc"]["_meta"] == {}
@@ -358,14 +359,14 @@ class TestRoot(object):
         nodes = {"no_table_specified": "book", "columns": ["description"]}
         sync.nodes = nodes
         with pytest.raises(TableNotInNodeError):
-            [doc for doc in sync._sync()]
+            [doc for doc in sync.sync()]
 
     def test_node_valid_attributes(self, sync, data):
         """All node must have valid attributes."""
         nodes = {"table": "book", "unknown": "xyz", "columns": ["description"]}
         sync.nodes = nodes
         with pytest.raises(NodeAttributeError):
-            [doc for doc in sync._sync()]
+            [doc for doc in sync.sync()]
 
     def test_update_primary_key_non_concurrent(self, data, book_cls):
         """
@@ -380,7 +381,7 @@ class TestRoot(object):
             "nodes": {"table": "book", "columns": ["isbn", "title"]},
         }
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
@@ -399,7 +400,7 @@ class TestRoot(object):
                 .values(isbn="cba")
             )
 
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
@@ -411,6 +412,7 @@ class TestRoot(object):
             {"_meta": {}, "isbn": "ghi", "title": "The Rabbit Club"},
         ]
         assert_resync_empty(sync, document.get("node", {}))
+        sync.es.close()
 
     # TODO: Add another test like this and change
     # both primary key and non primary key column
@@ -421,7 +423,7 @@ class TestRoot(object):
             "nodes": {"table": "book", "columns": ["isbn", "title"]},
         }
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
@@ -456,10 +458,14 @@ class TestRoot(object):
                 with mock.patch("pgsync.sync.Sync.pull", side_effect=pull):
                     with mock.patch(
                         "pgsync.sync.Sync.truncate_slots",
-                        side_effect=truncate_slots,
+                        side_effect=noop,
                     ):
-                        sync.receive()
-                        sync.es.refresh("testdb")
+                        with mock.patch(
+                            "pgsync.sync.Sync.status",
+                            side_effect=noop,
+                        ):
+                            sync.receive()
+                            sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
 
@@ -473,6 +479,7 @@ class TestRoot(object):
             {"_meta": {}, "isbn": "ghi", "title": "The Rabbit Club"},
         ]
         assert_resync_empty(sync, document.get("node", {}))
+        sync.es.close()
 
     def test_insert_non_concurrent(self, data, book_cls):
         """Test sync insert and then sync in non-concurrent mode."""
@@ -481,7 +488,7 @@ class TestRoot(object):
             "nodes": {"table": "book", "columns": ["isbn", "title"]},
         }
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         session = sync.session
@@ -500,7 +507,7 @@ class TestRoot(object):
                 )
             )
 
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
@@ -512,6 +519,7 @@ class TestRoot(object):
             {"_meta": {}, "isbn": "xyz", "title": "Encyclopedia"},
         ]
         assert_resync_empty(sync, document.get("node", {}))
+        sync.es.close()
 
     def test_update_non_concurrent(self, data, book_cls):
         """Test sync update and then sync in non-concurrent mode."""
@@ -520,7 +528,7 @@ class TestRoot(object):
             "nodes": {"table": "book", "columns": ["isbn", "title"]},
         }
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         session = sync.session
@@ -540,7 +548,7 @@ class TestRoot(object):
                 .values(title="Tiger Club")
             )
 
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
@@ -551,6 +559,7 @@ class TestRoot(object):
             {"_meta": {}, "isbn": "ghi", "title": "The Rabbit Club"},
         ]
         assert_resync_empty(sync, document.get("node", {}))
+        sync.es.close()
 
     def test_update_concurrent(self, data, book_cls):
         """Test sync update and then sync in concurrent mode."""
@@ -559,7 +568,7 @@ class TestRoot(object):
             "nodes": {"table": "book", "columns": ["isbn", "title"]},
         }
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         session = sync.session
@@ -594,10 +603,14 @@ class TestRoot(object):
                 with mock.patch("pgsync.sync.Sync.pull", side_effect=pull):
                     with mock.patch(
                         "pgsync.sync.Sync.truncate_slots",
-                        side_effect=truncate_slots,
+                        side_effect=noop,
                     ):
-                        sync.receive()
-                        sync.es.refresh("testdb")
+                        with mock.patch(
+                            "pgsync.sync.Sync.status",
+                            side_effect=noop,
+                        ):
+                            sync.receive()
+                            sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
 
@@ -607,6 +620,7 @@ class TestRoot(object):
             {"_meta": {}, "isbn": "ghi", "title": "The Rabbit Club"},
         ]
         assert_resync_empty(sync, document.get("node", {}))
+        sync.es.close()
 
     def test_delete_concurrent(self, data, book_cls):
         """Test sync delete and then sync in concurrent mode."""
@@ -615,7 +629,7 @@ class TestRoot(object):
             "nodes": {"table": "book", "columns": ["isbn", "title"]},
         }
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
         session = sync.session
         docs = search(sync.es, "testdb")
@@ -648,10 +662,14 @@ class TestRoot(object):
                 with mock.patch("pgsync.sync.Sync.pull", side_effect=pull):
                     with mock.patch(
                         "pgsync.sync.Sync.truncate_slots",
-                        side_effect=truncate_slots,
+                        side_effect=noop,
                     ):
-                        sync.receive()
-                        sync.es.refresh("testdb")
+                        with mock.patch(
+                            "pgsync.sync.Sync.status",
+                            side_effect=noop,
+                        ):
+                            sync.receive()
+                            sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
 
@@ -660,6 +678,7 @@ class TestRoot(object):
             {"_meta": {}, "isbn": "ghi", "title": "The Rabbit Club"},
         ]
         assert_resync_empty(sync, document.get("node", {}))
+        sync.es.close()
 
     def test_truncate(self, data, book_cls):
         """Test truncate."""
