@@ -15,7 +15,7 @@ from pgsync.exc import (
 from pgsync.node import Tree
 from pgsync.sync import Sync
 
-from .helpers.utils import assert_resync_empty, search, truncate_slots
+from .helpers.utils import assert_resync_empty, noop, search
 
 
 @pytest.mark.usefixtures("table_creator")
@@ -94,6 +94,7 @@ class TestParentSingleChildFkOnChild(object):
         sync.redis._delete()
         session.connection().engine.connect().close()
         session.connection().engine.dispose()
+        sync.es.close()
 
     def test_relationship_object_one_to_one(self, sync, data):
         nodes = {
@@ -111,7 +112,7 @@ class TestParentSingleChildFkOnChild(object):
             ],
         }
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
             "_meta": {"rating": {"id": [1]}},
@@ -156,7 +157,8 @@ class TestParentSingleChildFkOnChild(object):
             ],
         }
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
+        docs = sorted(docs, key=lambda k: k["_id"])
 
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
@@ -202,7 +204,7 @@ class TestParentSingleChildFkOnChild(object):
             ],
         }
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
             "_meta": {"rating": {"id": [1]}},
@@ -247,7 +249,8 @@ class TestParentSingleChildFkOnChild(object):
             ],
         }
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
+        docs = sorted(docs, key=lambda k: k["_id"])
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
             "_meta": {"rating": {"id": [1]}},
@@ -293,7 +296,7 @@ class TestParentSingleChildFkOnChild(object):
             ],
         }
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
             "_meta": {"rating": {"id": [1]}},
@@ -340,7 +343,7 @@ class TestParentSingleChildFkOnChild(object):
             ],
         }
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
             "_meta": {"rating": {"id": [1]}},
@@ -373,7 +376,7 @@ class TestParentSingleChildFkOnChild(object):
             ],
         }
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
 
         assert docs[0]["_source"] == {
@@ -418,7 +421,7 @@ class TestParentSingleChildFkOnChild(object):
             ],
         }
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
 
         fields = ["_meta", "description", "isbn", "rating", "title"]
         assert sorted(docs[0]["_source"].keys()) == sorted(fields)
@@ -445,7 +448,7 @@ class TestParentSingleChildFkOnChild(object):
             ],
         }
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         assert docs[2]["_source"] == {
             "_meta": {"rating": {"id": [3]}},
             "copyright": None,
@@ -467,6 +470,7 @@ class TestParentSingleChildFkOnChild(object):
                 }
             ],
         }
+        sync.es.close()
         with pytest.raises(RelationshipTypeError) as excinfo:
             Tree(sync).build(nodes)
         assert 'Relationship type "qwerty" is invalid' in str(excinfo.value)
@@ -484,6 +488,7 @@ class TestParentSingleChildFkOnChild(object):
                 }
             ],
         }
+        sync.es.close()
         with pytest.raises(RelationshipVariantError) as excinfo:
             Tree(sync).build(nodes)
         assert 'Relationship variant "abcdefg" is invalid' in str(
@@ -500,6 +505,7 @@ class TestParentSingleChildFkOnChild(object):
                 }
             ],
         }
+        sync.es.close()
         with pytest.raises(RelationshipAttributeError) as excinfo:
             Tree(sync).build(nodes)
         assert f"Relationship attribute {set(['foo'])} is invalid" in str(
@@ -521,7 +527,7 @@ class TestParentSingleChildFkOnChild(object):
             ],
         }
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         sources = {doc["_id"]: doc["_source"] for doc in docs}
         assert sources["abc"]["_meta"] == {"rating": {"id": [1]}}
         assert sources["def"]["_meta"] == {"rating": {"id": [2]}}
@@ -544,7 +550,7 @@ class TestParentSingleChildFkOnChild(object):
         }
         sync.nodes = nodes
         with pytest.raises(ForeignKeyError) as excinfo:
-            [doc for doc in sync._sync()]
+            [doc for doc in sync.sync()]
         msg = (
             "No foreign key relationship between "
             '"public.book" and "public.city"'
@@ -556,7 +562,7 @@ class TestParentSingleChildFkOnChild(object):
         nodes = {"table": "book", "children": [{"table": "rating"}]}
         sync.nodes = nodes
         with pytest.raises(RelationshipError) as excinfo:
-            [doc for doc in sync._sync()]
+            [doc for doc in sync.sync()]
         assert 'Relationship not present on "public.rating"' in str(
             excinfo.value
         )
@@ -585,7 +591,7 @@ class TestParentSingleChildFkOnChild(object):
             },
         }
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
@@ -628,7 +634,7 @@ class TestParentSingleChildFkOnChild(object):
             session.rollback()
             raise
 
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
@@ -659,6 +665,7 @@ class TestParentSingleChildFkOnChild(object):
             },
         ]
         assert_resync_empty(sync, document.get("node", {}))
+        sync.es.close()
 
     # TODO: Add another test like this and change
     # both primary key and non pkey column
@@ -682,7 +689,7 @@ class TestParentSingleChildFkOnChild(object):
             },
         }
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
@@ -741,10 +748,14 @@ class TestParentSingleChildFkOnChild(object):
                 with mock.patch("pgsync.sync.Sync.pull", side_effect=pull):
                     with mock.patch(
                         "pgsync.sync.Sync.truncate_slots",
-                        side_effect=truncate_slots,
+                        side_effect=noop,
                     ):
-                        sync.receive()
-                        sync.es.refresh("testdb")
+                        with mock.patch(
+                            "pgsync.sync.Sync.status",
+                            side_effect=noop,
+                        ):
+                            sync.receive()
+                            sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
         assert docs == [
@@ -774,6 +785,7 @@ class TestParentSingleChildFkOnChild(object):
             },
         ]
         assert_resync_empty(sync, document.get("node", {}))
+        sync.es.close()
 
     def test_insert_non_concurrent(self, data, book_cls, rating_cls):
         """Test sync insert and then sync in non-concurrent mode."""
@@ -795,7 +807,7 @@ class TestParentSingleChildFkOnChild(object):
             },
         }
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         session = sync.session
@@ -841,7 +853,7 @@ class TestParentSingleChildFkOnChild(object):
             session.rollback()
             raise
 
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
@@ -873,6 +885,7 @@ class TestParentSingleChildFkOnChild(object):
             },
         ]
         assert_resync_empty(sync, document.get("node", {}))
+        sync.es.close()
 
     def test_update_non_primary_key_non_concurrent(
         self, data, book_cls, rating_cls
@@ -896,7 +909,7 @@ class TestParentSingleChildFkOnChild(object):
             },
         }
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
@@ -935,7 +948,7 @@ class TestParentSingleChildFkOnChild(object):
             session.rollback()
             raise
 
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
@@ -961,6 +974,7 @@ class TestParentSingleChildFkOnChild(object):
             },
         ]
         assert_resync_empty(sync, document.get("node", {}))
+        sync.es.close()
 
     def test_update_non_primary_key_concurrent(
         self, data, book_cls, rating_cls
@@ -984,7 +998,7 @@ class TestParentSingleChildFkOnChild(object):
             },
         }
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
@@ -1034,10 +1048,14 @@ class TestParentSingleChildFkOnChild(object):
                 with mock.patch("pgsync.sync.Sync.pull", side_effect=pull):
                     with mock.patch(
                         "pgsync.sync.Sync.truncate_slots",
-                        side_effect=truncate_slots,
+                        side_effect=noop,
                     ):
-                        sync.receive()
-                        sync.es.refresh("testdb")
+                        with mock.patch(
+                            "pgsync.sync.Sync.status",
+                            side_effect=noop,
+                        ):
+                            sync.receive()
+                            sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
 
@@ -1062,6 +1080,7 @@ class TestParentSingleChildFkOnChild(object):
             },
         ]
         assert_resync_empty(sync, document.get("node", {}))
+        sync.es.close()
 
     def test_delete_concurrent(self, data, book_cls, rating_cls):
         """Test sync delete and then sync in concurrent mode."""
@@ -1084,7 +1103,7 @@ class TestParentSingleChildFkOnChild(object):
         }
 
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
@@ -1148,10 +1167,14 @@ class TestParentSingleChildFkOnChild(object):
                 with mock.patch("pgsync.sync.Sync.pull", side_effect=pull):
                     with mock.patch(
                         "pgsync.sync.Sync.truncate_slots",
-                        side_effect=truncate_slots,
+                        side_effect=noop,
                     ):
-                        sync.receive()
-                        sync.es.refresh("testdb")
+                        with mock.patch(
+                            "pgsync.sync.Sync.status",
+                            side_effect=noop,
+                        ):
+                            sync.receive()
+                            sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
         assert docs == [
@@ -1175,3 +1198,4 @@ class TestParentSingleChildFkOnChild(object):
             },
         ]
         assert_resync_empty(sync, document.get("node", {}))
+        sync.es.close()
