@@ -15,7 +15,7 @@ from pgsync.exc import (
 from pgsync.node import Tree
 from pgsync.sync import Sync
 
-from .helpers.utils import assert_resync_empty, search, truncate_slots
+from .helpers.utils import assert_resync_empty, noop, search
 
 
 @pytest.mark.usefixtures("table_creator")
@@ -85,6 +85,7 @@ class TestParentSingleChildFkOnParent(object):
         sync.redis._delete()
         session.connection().engine.connect().close()
         session.connection().engine.dispose()
+        sync.es.close()
 
     def test_relationship_object_one_to_one(self, sync, data):
         nodes = {
@@ -102,7 +103,7 @@ class TestParentSingleChildFkOnParent(object):
             ],
         }
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
             "_meta": {"publisher": {"id": [1]}},
@@ -147,7 +148,8 @@ class TestParentSingleChildFkOnParent(object):
             ],
         }
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
+        docs = sorted(docs, key=lambda k: k["_id"])
 
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
@@ -193,7 +195,7 @@ class TestParentSingleChildFkOnParent(object):
             ],
         }
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
             "_meta": {"publisher": {"id": [1]}},
@@ -238,7 +240,8 @@ class TestParentSingleChildFkOnParent(object):
             ],
         }
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
+        docs = sorted(docs, key=lambda k: k["_id"])
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
             "_meta": {"publisher": {"id": [1]}},
@@ -284,7 +287,7 @@ class TestParentSingleChildFkOnParent(object):
             ],
         }
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
             "_meta": {"publisher": {"id": [1]}},
@@ -331,7 +334,7 @@ class TestParentSingleChildFkOnParent(object):
             ],
         }
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
             "_meta": {"publisher": {"id": [1]}},
@@ -367,7 +370,7 @@ class TestParentSingleChildFkOnParent(object):
             ],
         }
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         assert docs[0]["_id"] == "abc"
         assert docs[0]["_source"] == {
             "_meta": {"publisher": {"id": [1]}},
@@ -421,7 +424,7 @@ class TestParentSingleChildFkOnParent(object):
             ],
         }
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
 
         fields = ["_meta", "description", "isbn", "publisher", "title"]
         assert sorted(docs[0]["_source"].keys()) == sorted(fields)
@@ -448,7 +451,7 @@ class TestParentSingleChildFkOnParent(object):
             ],
         }
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         assert docs[2]["_source"] == {
             "_meta": {"publisher": {"id": [3]}},
             "copyright": None,
@@ -470,6 +473,7 @@ class TestParentSingleChildFkOnParent(object):
                 }
             ],
         }
+        sync.es.close()
         with pytest.raises(RelationshipTypeError) as excinfo:
             Tree(sync).build(nodes)
         assert 'Relationship type "qwerty" is invalid' in str(excinfo.value)
@@ -487,6 +491,7 @@ class TestParentSingleChildFkOnParent(object):
                 }
             ],
         }
+        sync.es.close()
         with pytest.raises(RelationshipVariantError) as excinfo:
             Tree(sync).build(nodes)
         assert 'Relationship variant "abcdefg" is invalid' in str(
@@ -503,6 +508,7 @@ class TestParentSingleChildFkOnParent(object):
                 }
             ],
         }
+        sync.es.close()
         with pytest.raises(RelationshipAttributeError) as excinfo:
             Tree(sync).build(nodes)
         assert f"Relationship attribute {set(['foo'])} is invalid" in str(
@@ -524,7 +530,7 @@ class TestParentSingleChildFkOnParent(object):
             ],
         }
         sync.nodes = nodes
-        docs = [doc for doc in sync._sync()]
+        docs = [doc for doc in sync.sync()]
         sources = {doc["_id"]: doc["_source"] for doc in docs}
         assert sources["abc"]["_meta"] == {"publisher": {"id": [1]}}
         assert sources["def"]["_meta"] == {"publisher": {"id": [2]}}
@@ -547,7 +553,7 @@ class TestParentSingleChildFkOnParent(object):
         }
         sync.nodes = nodes
         with pytest.raises(ForeignKeyError) as excinfo:
-            [doc for doc in sync._sync()]
+            [doc for doc in sync.sync()]
         msg = (
             "No foreign key relationship between "
             '"public.book" and "public.city"'
@@ -559,7 +565,7 @@ class TestParentSingleChildFkOnParent(object):
         nodes = {"table": "book", "children": [{"table": "publisher"}]}
         sync.nodes = nodes
         with pytest.raises(RelationshipError) as excinfo:
-            [doc for doc in sync._sync()]
+            [doc for doc in sync.sync()]
         assert 'Relationship not present on "public.publisher"' in str(
             excinfo.value
         )
@@ -588,7 +594,7 @@ class TestParentSingleChildFkOnParent(object):
             },
         }
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
@@ -631,7 +637,7 @@ class TestParentSingleChildFkOnParent(object):
             session.rollback()
             raise
 
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
@@ -657,6 +663,7 @@ class TestParentSingleChildFkOnParent(object):
             },
         ]
         assert_resync_empty(sync, document.get("node", {}))
+        sync.es.close()
 
     # TODO: Add another test like this and change
     # both primary key and non pkey column
@@ -682,7 +689,7 @@ class TestParentSingleChildFkOnParent(object):
             },
         }
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
@@ -740,10 +747,14 @@ class TestParentSingleChildFkOnParent(object):
                 with mock.patch("pgsync.sync.Sync.pull", side_effect=pull):
                     with mock.patch(
                         "pgsync.sync.Sync.truncate_slots",
-                        side_effect=truncate_slots,
+                        side_effect=noop,
                     ):
-                        sync.receive()
-                        sync.es.refresh("testdb")
+                        with mock.patch(
+                            "pgsync.sync.Sync.status",
+                            side_effect=noop,
+                        ):
+                            sync.receive()
+                            sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
 
@@ -768,6 +779,7 @@ class TestParentSingleChildFkOnParent(object):
             },
         ]
         assert_resync_empty(sync, document.get("node", {}))
+        sync.es.close()
 
     def test_insert_non_concurrent(self, data, book_cls, publisher_cls):
         """Test sync insert and then sync in non-concurrent mode."""
@@ -789,7 +801,7 @@ class TestParentSingleChildFkOnParent(object):
             },
         }
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         session = sync.session
@@ -832,7 +844,7 @@ class TestParentSingleChildFkOnParent(object):
             session.rollback()
             raise
 
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
@@ -864,6 +876,7 @@ class TestParentSingleChildFkOnParent(object):
             },
         ]
         assert_resync_empty(sync, document.get("node", {}))
+        sync.es.close()
 
     def test_update_non_primary_key_non_concurrent(
         self, data, book_cls, publisher_cls
@@ -887,7 +900,7 @@ class TestParentSingleChildFkOnParent(object):
             },
         }
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
@@ -926,7 +939,7 @@ class TestParentSingleChildFkOnParent(object):
             session.rollback()
             raise
 
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
@@ -952,6 +965,7 @@ class TestParentSingleChildFkOnParent(object):
             },
         ]
         assert_resync_empty(sync, document.get("node", {}))
+        sync.es.close()
 
     def test_update_non_primary_key_concurrent(
         self, data, book_cls, publisher_cls
@@ -975,7 +989,7 @@ class TestParentSingleChildFkOnParent(object):
             },
         }
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
@@ -1025,10 +1039,14 @@ class TestParentSingleChildFkOnParent(object):
                 with mock.patch("pgsync.sync.Sync.pull", side_effect=pull):
                     with mock.patch(
                         "pgsync.sync.Sync.truncate_slots",
-                        side_effect=truncate_slots,
+                        side_effect=noop,
                     ):
-                        sync.receive()
-                        sync.es.refresh("testdb")
+                        with mock.patch(
+                            "pgsync.sync.Sync.status",
+                            side_effect=noop,
+                        ):
+                            sync.receive()
+                            sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
 
@@ -1053,6 +1071,7 @@ class TestParentSingleChildFkOnParent(object):
             },
         ]
         assert_resync_empty(sync, document.get("node", {}))
+        sync.es.close()
 
     def test_delete_concurrent(self, data, book_cls, publisher_cls):
         """Test sync delete and then sync in concurrent mode."""
@@ -1075,7 +1094,7 @@ class TestParentSingleChildFkOnParent(object):
         }
 
         sync = Sync(document)
-        sync.sync(sync._sync())
+        sync.es.bulk(sync.index, sync.sync())
         sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
@@ -1138,10 +1157,14 @@ class TestParentSingleChildFkOnParent(object):
                 with mock.patch("pgsync.sync.Sync.pull", side_effect=pull):
                     with mock.patch(
                         "pgsync.sync.Sync.truncate_slots",
-                        side_effect=truncate_slots,
+                        side_effect=noop,
                     ):
-                        sync.receive()
-                        sync.es.refresh("testdb")
+                        with mock.patch(
+                            "pgsync.sync.Sync.status",
+                            side_effect=noop,
+                        ):
+                            sync.receive()
+                            sync.es.refresh("testdb")
 
         docs = search(sync.es, "testdb")
 
@@ -1166,3 +1189,4 @@ class TestParentSingleChildFkOnParent(object):
             },
         ]
         assert_resync_empty(sync, document.get("node", {}))
+        sync.es.close()
