@@ -109,6 +109,8 @@ class TestTransform(object):
                 "id",
                 "code",
                 "level",
+                "foo",
+                "bar",
             ],
             "children": [
                 {
@@ -143,6 +145,8 @@ class TestTransform(object):
                     "id": "my_id",
                     "code": "my_code",
                     "level": "levelup",
+                    "foo": "foos",
+                    "bar": "bars",
                 }
             },
         }
@@ -151,6 +155,8 @@ class TestTransform(object):
             "level": 1,
             "id": "007",
             "code": "be",
+            "foo": ["a", "b"],
+            "bar": {"a": 1, "b": 2},
             "Child1": [
                 {"column_1": 2, "column_2": "aa"},
                 {"column_1": 3, "column_2": "bb"},
@@ -173,6 +179,8 @@ class TestTransform(object):
             "levelup": 1,
             "my_code": "be",
             "my_id": "007",
+            "foos": ["a", "b"],
+            "bars": {"a": 1, "b": 2},
         }
 
     def test_rename_fields(self):
@@ -250,10 +258,28 @@ class TestTransform(object):
                         "type": "one_to_many",
                     },
                     "transform": {
-                        "concat": {
-                            "columns": ["http", "://", "column1", "/", "path"],
-                            "destination": "column3",
-                        }
+                        "concat": [
+                            {
+                                "columns": [
+                                    "http",
+                                    "://",
+                                    "column1",
+                                    "/",
+                                    "path",
+                                ],
+                                "destination": "column3",
+                            },
+                            {
+                                "columns": [
+                                    "http",
+                                    "://",
+                                    "column1",
+                                    "/",
+                                    "path",
+                                ],
+                                "destination": "column4",
+                            },
+                        ]
                     },
                 },
                 {
@@ -308,8 +334,18 @@ class TestTransform(object):
                 {"column_1": 3, "column_2": "bb", "column_3": "3_bb"},
             ],
             "Child2": [
-                {"column1": 2, "column2": "cc", "column3": "http://2/path"},
-                {"column1": 3, "column2": "dd", "column3": "http://3/path"},
+                {
+                    "column1": 2,
+                    "column2": "cc",
+                    "column3": "http://2/path",
+                    "column4": "http://2/path",
+                },
+                {
+                    "column1": 3,
+                    "column2": "dd",
+                    "column3": "http://3/path",
+                    "column4": "http://3/path",
+                },
             ],
             "Child3": {"column_1": 4, "column_2": "ee", "column_9": "4@ee"},
             "code": "be",
